@@ -306,24 +306,36 @@ def options_zsh_join(optional_actions):
 def complete_zsh(parser, root_prefix=None, preamble="", choice_functions=None):
     root_prefix = "_shtab_" + (root_prefix or parser.prog)
 
-    # {cmd: {"help": help, "arguments": [arguments]}}
-    subcommands = {}
+    root_arguments = []
+    subcommands = {}  # {cmd: {"help": help, "arguments": [arguments]}}
 
     choice_type2fn = dict(CHOICE_FUNCTIONS_ZSH)
     if choice_functions:
         choice_type2fn.update(choice_functions)
 
-    if len(parser._get_positional_actions()) != 1:
-        logger.warning(
-            "NotImplementedError:Expecting one positional subparser"
-        )
     for sub in parser._get_positional_actions():
-        if sub.choices:
+        if not sub.choices or not isinstance(sub.choices, dict):
+            # positional argument
+            opt = sub
+            root_arguments.append(
+                '"{nargs}:{help}:{choices}"'.format(
+                    nargs={"+": "*", "*": "*"}.get(opt.nargs, ""),
+                    help=escape_zsh(
+                        (opt.help or opt.dest).strip().split("\n")[0]
+                    ),
+                    choices=(
+                        choice_type2fn[opt.choices[0].type]
+                        if isinstance(opt.choices[0], Choice)
+                        else "({})".format(" ".join(opt.choices))
+                    )
+                    if opt.choices
+                    else "",
+                )
+            )
+        else:  # subparser
             logger.debug(
                 "choices:{}:{}".format(root_prefix, sorted(sub.choices))
             )
-            if not isinstance(sub.choices, dict):
-                logger.warning("NotImplementedError:Expecting subparser dict")
             for cmd, subparser in sub.choices.items():
                 # optionals
                 arguments = [
@@ -371,12 +383,12 @@ def complete_zsh(parser, root_prefix=None, preamble="", choice_functions=None):
 
                 # positionals
                 arguments.extend(
-                    '"{}:{}:{}"'.format(
-                        {"+": "*", "*": "*"}.get(opt.nargs, ""),
-                        escape_zsh(
+                    '"{nargs}:{help}:{choices}"'.format(
+                        nargs={"+": "*", "*": "*"}.get(opt.nargs, ""),
+                        help=escape_zsh(
                             (opt.help or opt.dest).strip().split("\n")[0]
                         ),
-                        (
+                        choices=(
                             choice_type2fn[opt.choices[0].type]
                             if isinstance(opt.choices[0], Choice)
                             else "({})".format(" ".join(opt.choices))
@@ -395,8 +407,6 @@ def complete_zsh(parser, root_prefix=None, preamble="", choice_functions=None):
                     "arguments": arguments,
                 }
                 logger.debug("subcommands:%s:%s", cmd, subcommands[cmd])
-        else:
-            logger.warning("NotImplementedError:Expecting subparser choices")
 
     logger.debug("subcommands:%s:%s", root_prefix, sorted(subcommands))
 
@@ -413,7 +423,7 @@ def complete_zsh(parser, root_prefix=None, preamble="", choice_functions=None):
 # AUTOMATCALLY GENERATED by `shtab`
 
 {root_prefix}_options_=(
-  {options}
+  {root_options}
 )
 
 {root_prefix}_commands_() {
@@ -430,7 +440,8 @@ local context state line curcontext="$curcontext"
 
 _arguments \\
   ${root_prefix}_options_ \\
-  '1: :{root_prefix}_commands_' \\
+  {root_arguments} \\
+  ': :{root_prefix}_commands_' \\
   '*::args:->args'
 
 case $words[1] in
@@ -442,7 +453,8 @@ esac""",
             '"{}:{}"'.format(cmd, escape_zsh(subcommands[cmd]["help"]))
             for cmd in sorted(subcommands)
         ),
-        options="\n  ".join(
+        root_arguments=" \\\n  ".join(root_arguments),
+        root_options="\n  ".join(
             '{nargs}{options}"[{help}]"'.format(
                 nargs=(
                     '"(- :)"'
